@@ -282,6 +282,10 @@ function cleanup_procs() {
   fi
 
   if [[ "${DRIVER}" == "hyperkit" ]]; then
+    # even though Internet Sharing is disabled in the UI settings, it's still preventing HyperKit from starting
+    # the error is "Could not create vmnet interface, permission denied or no entitlement?"
+    # I've discovered that if you kill the "InternetSharing" process that this resolves the error and HyperKit starts normally
+    sudo pkill InternetSharing
     if [[ -e out/docker-machine-driver-hyperkit ]]; then
       sudo chown root:wheel out/docker-machine-driver-hyperkit || true
       sudo chmod u+s out/docker-machine-driver-hyperkit || true
@@ -341,7 +345,7 @@ if [ "$(uname)" != "Darwin" ]; then
   docker build -t gcr.io/k8s-minikube/gvisor-addon:2 -f testdata/gvisor-addon-Dockerfile ./testdata
 fi
 
-readonly LOAD=$(uptime | egrep -o "load average.*: [0-9]+" | cut -d" " -f3)
+readonly LOAD=$(uptime | grep -E -o "load average.*: [0-9]+" | cut -d" " -f3)
 if [[ "${LOAD}" -gt 2 ]]; then
   echo ""
   echo "********************** LOAD WARNING ********************************"
