@@ -25,13 +25,21 @@ import (
 	"k8s.io/minikube/pkg/minikube/out/register"
 )
 
-// currentLogFile the file that's used to store audit logs
-var currentLogFile *os.File
+var (
+	// currentLogFile the file that's used to store audit logs
+	currentLogFile *os.File
+
+	// auditOverrideFilename overrides the default audit log filename, used for testing purposes
+	auditOverrideFilename string
+)
 
 // openAuditLog opens the audit log file or creates it if it doesn't exist.
 func openAuditLog() error {
-	lp := localpath.AuditLog()
-	f, err := os.OpenFile(lp, os.O_APPEND|os.O_CREATE|os.O_RDWR, 0644)
+	// this is so we can manually set the log file for tests
+	if currentLogFile != nil {
+		return nil
+	}
+	f, err := os.OpenFile(auditPath(), os.O_APPEND|os.O_CREATE|os.O_RDWR, 0644)
 	if err != nil {
 		return fmt.Errorf("failed to open the audit log: %v", err)
 	}
@@ -44,6 +52,7 @@ func closeAuditLog() {
 	if err := currentLogFile.Close(); err != nil {
 		klog.Errorf("failed to close the audit log: %v", err)
 	}
+	currentLogFile = nil
 }
 
 // appendToLog appends the row to the log file.
@@ -62,3 +71,18 @@ func appendToLog(row *row) error {
 	}
 	return nil
 }
+
+// truncateAuditLog truncates the audit log file
+func truncateAuditLog() error {
+	if err := os.Truncate(auditPath(), 0); err != nil {
+		return fmt.Errorf("failed to truncate audit log: %v", err)
+	}
+	return nil
+}
+
+func auditPath() string {
+	if auditOverrideFilename != "" {
+		return auditOverrideFilename
+	}
+	return localpath.AuditLog()
+}
