@@ -22,6 +22,7 @@ import (
 	"fmt"
 	"io"
 	"net"
+	"os"
 	"os/exec"
 	"path"
 	"runtime"
@@ -48,6 +49,7 @@ import (
 	"k8s.io/minikube/pkg/minikube/bootstrapper/bsutil"
 	"k8s.io/minikube/pkg/minikube/bootstrapper/bsutil/kverify"
 	"k8s.io/minikube/pkg/minikube/bootstrapper/images"
+	kubevip "k8s.io/minikube/pkg/minikube/cluster/ha/kube-vip"
 	"k8s.io/minikube/pkg/minikube/cni"
 	"k8s.io/minikube/pkg/minikube/command"
 	"k8s.io/minikube/pkg/minikube/config"
@@ -136,8 +138,8 @@ func (k *Bootstrapper) createCompatSymlinks() error {
 	return nil
 }
 
-// clearStaleConfigs clears configurations which may have stale IP addresses
-func (k *Bootstrapper) clearStaleConfigs(cfg config.ClusterConfig) error {
+// clearStaleConfigs tries to clear configurations which may have stale IP addresses.
+func (k *Bootstrapper) clearStaleConfigs(cfg config.ClusterConfig) {
 	// These are the files that kubeadm will reject stale versions of
 	paths := []string{
 		"/etc/kubernetes/admin.conf",
@@ -150,16 +152,10 @@ func (k *Bootstrapper) clearStaleConfigs(cfg config.ClusterConfig) error {
 	rr, err := k.c.RunCmd(exec.Command("sudo", args...))
 	if err != nil {
 		klog.Infof("config check failed, skipping stale config cleanup: %v", err)
-		return nil
 	}
 	klog.Infof("found existing configuration files:\n%s\n", rr.Stdout.String())
 
-	cp, err := config.PrimaryControlPlane(&cfg)
-	if err != nil {
-		return err
-	}
-
-	endpoint := fmt.Sprintf("https://%s", net.JoinHostPort(constants.ControlPlaneAlias, strconv.Itoa(cp.Port)))
+	endpoint := fmt.Sprintf("https://%s", net.JoinHostPort(constants.ControlPlaneAlias, strconv.Itoa(cfg.APIServerPort)))
 	for _, path := range paths {
 		_, err := k.c.RunCmd(exec.Command("sudo", "grep", endpoint, path))
 		if err != nil {
@@ -171,9 +167,9 @@ func (k *Bootstrapper) clearStaleConfigs(cfg config.ClusterConfig) error {
 			}
 		}
 	}
-	return nil
 }
 
+// init initialises primary control-plane using kubeadm.
 func (k *Bootstrapper) init(cfg config.ClusterConfig) error {
 	version, err := util.ParseKubernetesVersion(cfg.KubernetesConfig.KubernetesVersion)
 	if err != nil {
@@ -196,11 +192,7 @@ func (k *Bootstrapper) init(cfg config.ClusterConfig) error {
 		"FileAvailable--etc-kubernetes-manifests-etcd.yaml",
 		"Port-10250", // For "none" users who already have a kubelet online
 		"Swap",       // For "none" users who have swap configured
-	}
-	if version.GE(semver.MustParse("1.13.0")) {
-		ignore = append(ignore,
-			"NumCPU", // For "none" users who have too few CPUs
-		)
+		"NumCPU",     // For "none" users who have too few CPUs
 	}
 	if version.GE(semver.MustParse("1.20.0")) {
 		ignore = append(ignore,
@@ -210,11 +202,6 @@ func (k *Bootstrapper) init(cfg config.ClusterConfig) error {
 	ignore = append(ignore, bsutil.SkipAdditionalPreflights[r.Name()]...)
 
 	skipSystemVerification := false
-	// Allow older kubeadm versions to function with newer Docker releases.
-	if version.LT(semver.MustParse("1.13.0")) {
-		klog.Infof("ignoring SystemVerification for kubeadm because of old Kubernetes version %v", version)
-		skipSystemVerification = true
-	}
 	if driver.BareMetal(cfg.Driver) && r.Name() == "Docker" {
 		if v, err := r.Version(); err == nil && strings.Contains(v, "azure") {
 			klog.Infof("ignoring SystemVerification for kubeadm because of unknown docker version %s", v)
@@ -234,9 +221,7 @@ func (k *Bootstrapper) init(cfg config.ClusterConfig) error {
 		ignore = append(ignore, "FileContent--proc-sys-net-bridge-bridge-nf-call-iptables")
 	}
 
-	if err := k.clearStaleConfigs(cfg); err != nil {
-		return errors.Wrap(err, "clearing stale configs")
-	}
+	k.clearStaleConfigs(cfg)
 
 	conf := constants.KubeadmYamlPath
 	ctx, cancel := context.WithTimeout(context.Background(), initTimeoutMinutes*time.Minute)
@@ -273,32 +258,34 @@ func (k *Bootstrapper) init(cfg config.ClusterConfig) error {
 	wg.Add(3)
 
 	go func() {
+		defer wg.Done()
 		// we need to have cluster role binding before applying overlay to avoid #7428
 		if err := k.elevateKubeSystemPrivileges(cfg); err != nil {
-			klog.Errorf("unable to create cluster role binding, some addons might not work: %v", err)
+			klog.Errorf("unable to create cluster role binding for primary control-plane node, some addons might not work: %v", err)
 		}
-		wg.Done()
 	}()
 
 	go func() {
-		if err := k.applyNodeLabels(cfg); err != nil {
-			klog.Warningf("unable to apply node labels: %v", err)
+		defer wg.Done()
+		if err := k.LabelAndUntaintNode(cfg, config.ControlPlanes(cfg)[0]); err != nil {
+			klog.Warningf("unable to apply primary control-plane node labels and taints: %v", err)
 		}
-		wg.Done()
 	}()
 
 	go func() {
+		defer wg.Done()
 		if err := bsutil.AdjustResourceLimits(k.c); err != nil {
-			klog.Warningf("unable to adjust resource limits: %v", err)
+			klog.Warningf("unable to adjust resource limits for primary control-plane node: %v", err)
 		}
-		wg.Done()
 	}()
 
 	wg.Wait()
-	// Tunnel apiserver to guest, if necessary
-	if cfg.APIServerPort != 0 {
-		k.tunnelToAPIServer(cfg)
+
+	// tunnel apiserver to guest
+	if err := k.tunnelToAPIServer(cfg); err != nil {
+		klog.Warningf("apiserver tunnel failed: %v", err)
 	}
+
 	return nil
 }
 
@@ -403,7 +390,7 @@ func (k *Bootstrapper) StartCluster(cfg config.ClusterConfig) error {
 	start := time.Now()
 	klog.Infof("StartCluster: %+v", cfg)
 	defer func() {
-		klog.Infof("StartCluster complete in %s", time.Since(start))
+		klog.Infof("duration metric: took %s to StartCluster", time.Since(start))
 	}()
 
 	// Before we start, ensure that no paused components are lurking around
@@ -412,17 +399,18 @@ func (k *Bootstrapper) StartCluster(cfg config.ClusterConfig) error {
 	}
 
 	if err := bsutil.ExistingConfig(k.c); err == nil {
-		// If the guest already exists and was stopped, re-establish the apiserver tunnel so checks pass
-		if cfg.APIServerPort != 0 {
-			k.tunnelToAPIServer(cfg)
-		}
-		klog.Infof("found existing configuration files, will attempt cluster restart")
-		rerr := k.restartControlPlane(cfg)
-		if rerr == nil {
-			return nil
+		// if the guest already exists and was stopped, re-establish the apiserver tunnel so checks pass
+		if err := k.tunnelToAPIServer(cfg); err != nil {
+			klog.Warningf("apiserver tunnel failed: %v", err)
 		}
 
-		out.ErrT(style.Embarrassed, "Unable to restart cluster, will reset it: {{.error}}", out.V{"error": rerr})
+		klog.Infof("found existing configuration files, will attempt cluster restart")
+
+		var rerr error
+		if rerr := k.restartPrimaryControlPlane(cfg); rerr == nil {
+			return nil
+		}
+		out.ErrT(style.Embarrassed, "Unable to restart control-plane node(s), will reset cluster: {{.error}}", out.V{"error": rerr})
 		if err := k.DeleteCluster(cfg.KubernetesConfig); err != nil {
 			klog.Warningf("delete failed: %v", err)
 		}
@@ -450,20 +438,27 @@ func (k *Bootstrapper) StartCluster(cfg config.ClusterConfig) error {
 	return err
 }
 
-func (k *Bootstrapper) tunnelToAPIServer(cfg config.ClusterConfig) {
+// tunnelToAPIServer creates ssh tunnel between apiserver:port inside control-plane node and host on port 8443.
+func (k *Bootstrapper) tunnelToAPIServer(cfg config.ClusterConfig) error {
+	if cfg.APIServerPort != 0 {
+		return fmt.Errorf("apiserver port not set")
+	}
+
 	m, err := machine.NewAPIClient()
 	if err != nil {
-		klog.Warningf("libmachine API failed: %v", err)
+		return errors.Wrapf(err, "create libmachine api client")
 	}
-	cp, err := config.PrimaryControlPlane(&cfg)
+
+	cp, err := config.ControlPlane(cfg)
 	if err != nil {
-		klog.Warningf("finding control plane failed: %v", err)
+		return errors.Wrapf(err, "get control-plane node")
 	}
+
 	args := []string{"-f", "-NTL", fmt.Sprintf("%d:localhost:8443", cfg.APIServerPort)}
-	err = machine.CreateSSHShell(m, cfg, cp, args, false)
-	if err != nil {
-		klog.Warningf("apiserver tunnel failed: %v", err)
+	if err = machine.CreateSSHShell(m, cfg, cp, args, false); err != nil {
+		return errors.Wrapf(err, "ssh command")
 	}
+	return nil
 }
 
 // client sets and returns a Kubernetes client to use to speak to a kubeadm launched apiserver
@@ -496,17 +491,17 @@ func (k *Bootstrapper) WaitForNode(cfg config.ClusterConfig, n config.Node, time
 	out.Step(style.HealthCheck, "Verifying Kubernetes components...")
 	// regardless if waiting is set or not, we will make sure kubelet is not stopped
 	// to solve corner cases when a container is hibernated and once coming back kubelet not running.
-	if err := k.ensureServiceStarted("kubelet"); err != nil {
+	if err := sysinit.New(k.c).Start("kubelet"); err != nil {
 		klog.Warningf("Couldn't ensure kubelet is started this might cause issues: %v", err)
 	}
 	// TODO: #7706: for better performance we could use k.client inside minikube to avoid asking for external IP:PORT
-	cp, err := config.PrimaryControlPlane(&cfg)
+	cp, err := config.ControlPlane(cfg)
 	if err != nil {
-		return errors.Wrap(err, "get primary control plane")
+		return errors.Wrap(err, "get control-plane node")
 	}
 	hostname, _, port, err := driver.ControlPlaneEndpoint(&cfg, &cp, cfg.Driver)
 	if err != nil {
-		return errors.Wrap(err, "get control plane endpoint")
+		return errors.Wrap(err, "get control-plane endpoint")
 	}
 
 	client, err := k.client(hostname, port)
@@ -578,7 +573,7 @@ func (k *Bootstrapper) WaitForNode(cfg config.ClusterConfig, n config.Node, time
 		}
 	}
 
-	klog.Infof("duration metric: took %s to wait for : %+v ...", time.Since(start), cfg.VerifyComponents)
+	klog.Infof("duration metric: took %s to wait for: %+v", time.Since(start), cfg.VerifyComponents)
 
 	if err := kverify.NodePressure(client); err != nil {
 		adviseNodePressure(err, cfg.Name, cfg.Driver)
@@ -587,92 +582,35 @@ func (k *Bootstrapper) WaitForNode(cfg config.ClusterConfig, n config.Node, time
 	return nil
 }
 
-// ensureServiceStarted will start a systemd or init.d service if it is not running.
-func (k *Bootstrapper) ensureServiceStarted(svc string) error {
-	if st := kverify.ServiceStatus(k.c, svc); st != state.Running {
-		klog.Warningf("surprisingly %q service status was %s!. will try to start it, could be related to this issue https://github.com/kubernetes/minikube/issues/9458", svc, st)
-		return sysinit.New(k.c).Start(svc)
-	}
-	return nil
-}
-
-// needsReconfigure returns whether or not the cluster needs to be reconfigured
-func (k *Bootstrapper) needsReconfigure(conf string, hostname string, port int, client *kubernetes.Clientset, version string) bool {
-	if rr, err := k.c.RunCmd(exec.Command("sudo", "diff", "-u", conf, conf+".new")); err != nil {
-		klog.Infof("needs reconfigure: configs differ:\n%s", rr.Output())
-		return true
-	}
-
-	// cruntime.Enable() may restart kube-apiserver but does not wait for it to return back
-	// could take five-ish seconds, so hopefully 10 seconds is sufficient to wait for api server to come back up
-	apiStatusTimeout := 10 * time.Second
-	st, err := kverify.WaitForAPIServerStatus(k.c, apiStatusTimeout, hostname, port)
-	if err != nil {
-		klog.Infof("needs reconfigure: apiserver error: %v", err)
-		return true
-	}
-	if st != state.Running {
-		klog.Infof("needs reconfigure: apiserver in state %s", st)
-		return true
-	}
-
-	if err := kverify.ExpectAppsRunning(client, kverify.AppsRunningList); err != nil {
-		klog.Infof("needs reconfigure: %v", err)
-		return true
-	}
-
-	if err := kverify.APIServerVersionMatch(client, version); err != nil {
-		klog.Infof("needs reconfigure: %v", err)
-		return true
-	}
-
-	// DANGER: This log message is hard-coded in an integration test!
-	klog.Infof("The running cluster does not require reconfiguration: %s", hostname)
-	return false
-}
-
-// restartCluster restarts the Kubernetes cluster configured by kubeadm
-func (k *Bootstrapper) restartControlPlane(cfg config.ClusterConfig) error {
-	klog.Infof("restartCluster start")
+// restartPrimaryControlPlane restarts the kubernetes cluster configured by kubeadm.
+func (k *Bootstrapper) restartPrimaryControlPlane(cfg config.ClusterConfig) error {
+	klog.Infof("restartPrimaryControlPlane start ...")
 
 	start := time.Now()
 	defer func() {
-		klog.Infof("restartCluster took %s", time.Since(start))
+		klog.Infof("duration metric: took %s to restartPrimaryControlPlane", time.Since(start))
 	}()
 
-	k8sVersion, err := util.ParseKubernetesVersion(cfg.KubernetesConfig.KubernetesVersion)
-	if err != nil {
-		return errors.Wrap(err, "parsing Kubernetes version")
-	}
-
-	phase := "alpha"
-	controlPlane := "controlplane"
-	if k8sVersion.GTE(semver.MustParse("1.13.0")) {
-		phase = "init"
-		controlPlane = "control-plane"
-	}
-
 	if err := k.createCompatSymlinks(); err != nil {
 		klog.Errorf("failed to create compat symlinks: %v", err)
 	}
 
-	cp, err := config.PrimaryControlPlane(&cfg)
-	if err != nil {
-		return errors.Wrap(err, "primary control plane")
+	pcp, err := config.ControlPlane(cfg)
+	if err != nil || !config.IsPrimaryControlPlane(cfg, pcp) {
+		return errors.Wrap(err, "get primary control-plane node")
 	}
 
-	hostname, _, port, err := driver.ControlPlaneEndpoint(&cfg, &cp, cfg.Driver)
+	host, _, port, err := driver.ControlPlaneEndpoint(&cfg, &pcp, cfg.Driver)
 	if err != nil {
-		return errors.Wrap(err, "control plane")
+		return errors.Wrap(err, "get primary control-plane endpoint")
 	}
 
 	// Save the costly tax of reinstalling Kubernetes if the only issue is a missing kube context
-	_, err = kubeconfig.UpdateEndpoint(cfg.Name, hostname, port, kubeconfig.PathFromEnv(), kubeconfig.NewExtension())
-	if err != nil {
+	if _, err := kubeconfig.UpdateEndpoint(cfg.Name, host, port, kubeconfig.PathFromEnv(), kubeconfig.NewExtension()); err != nil {
 		klog.Warningf("unable to update kubeconfig (cluster will likely require a reset): %v", err)
 	}
 
-	client, err := k.client(hostname, port)
+	client, err := k.client(host, port)
 	if err != nil {
 		return errors.Wrap(err, "getting k8s client")
 	}
@@ -680,37 +618,45 @@ func (k *Bootstrapper) restartControlPlane(cfg config.ClusterConfig) error {
 	// If the cluster is running, check if we have any work to do.
 	conf := constants.KubeadmYamlPath
 
-	if !k.needsReconfigure(conf, hostname, port, client, cfg.KubernetesConfig.KubernetesVersion) {
-		klog.Infof("Taking a shortcut, as the cluster seems to be properly configured")
-		return nil
+	// check whether or not the cluster needs to be reconfigured
+	if rr, err := k.c.RunCmd(exec.Command("sudo", "diff", "-u", conf, conf+".new")); err == nil {
+		// DANGER: This log message is hard-coded in an integration test!
+		klog.Infof("The running cluster does not require reconfiguration: %s", host)
+		// taking a shortcut, as the cluster seems to be properly configured
+		// except for vm driver in non-ha (non-multi-control plane) cluster - fallback to old behaviour
+		// here we're making a tradeoff to avoid significant (10sec) waiting on restarting stopped non-ha (non-multi-control plane) cluster with vm driver
+		// where such cluster needs to be reconfigured b/c of (currently) ephemeral config, but then also,
+		// starting already started such cluster (hard to know w/o investing that time) will fallthrough the same path and reconfigure cluster
+		if config.IsHA(cfg) || !driver.IsVM(cfg.Driver) {
+			return nil
+		}
+	} else {
+		klog.Infof("detected kubeadm config drift (will reconfigure cluster from new %s):\n%s", conf, rr.Output())
 	}
 
 	if err := k.stopKubeSystem(cfg); err != nil {
-		klog.Warningf("Failed to stop kube-system containers: port conflicts may arise: %v", err)
+		klog.Warningf("Failed to stop kube-system containers, port conflicts may arise: %v", err)
 	}
 
 	if err := sysinit.New(k.c).Stop("kubelet"); err != nil {
 		klog.Warningf("Failed to stop kubelet, this might cause upgrade errors: %v", err)
 	}
 
-	if err := k.clearStaleConfigs(cfg); err != nil {
-		return errors.Wrap(err, "clearing stale configs")
-	}
+	k.clearStaleConfigs(cfg)
 
 	if _, err := k.c.RunCmd(exec.Command("sudo", "cp", conf+".new", conf)); err != nil {
 		return errors.Wrap(err, "cp")
 	}
 
-	baseCmd := fmt.Sprintf("%s %s", bsutil.InvokeKubeadm(cfg.KubernetesConfig.KubernetesVersion), phase)
+	baseCmd := fmt.Sprintf("%s init", bsutil.InvokeKubeadm(cfg.KubernetesConfig.KubernetesVersion))
 	cmds := []string{
 		fmt.Sprintf("%s phase certs all --config %s", baseCmd, conf),
 		fmt.Sprintf("%s phase kubeconfig all --config %s", baseCmd, conf),
 		fmt.Sprintf("%s phase kubelet-start --config %s", baseCmd, conf),
-		fmt.Sprintf("%s phase %s all --config %s", baseCmd, controlPlane, conf),
+		fmt.Sprintf("%s phase control-plane all --config %s", baseCmd, conf),
 		fmt.Sprintf("%s phase etcd local --config %s", baseCmd, conf),
 	}
 
-	klog.Infof("reconfiguring cluster from %s", conf)
 	// Run commands one at a time so that it is easier to root cause failures.
 	for _, c := range cmds {
 		if _, err := k.c.RunCmd(exec.Command("/bin/bash", "-c", c)); err != nil {
@@ -732,7 +678,7 @@ func (k *Bootstrapper) restartControlPlane(cfg config.ClusterConfig) error {
 		return errors.Wrap(err, "apiserver healthz")
 	}
 
-	if err := kverify.WaitForHealthyAPIServer(cr, k, cfg, k.c, client, time.Now(), hostname, port, kconst.DefaultControlPlaneTimeout); err != nil {
+	if err := kverify.WaitForHealthyAPIServer(cr, k, cfg, k.c, client, time.Now(), host, port, kconst.DefaultControlPlaneTimeout); err != nil {
 		return errors.Wrap(err, "apiserver health")
 	}
 
@@ -801,9 +747,20 @@ func (k *Bootstrapper) restartControlPlane(cfg config.ClusterConfig) error {
 
 // JoinCluster adds new node to an existing cluster.
 func (k *Bootstrapper) JoinCluster(cc config.ClusterConfig, n config.Node, joinCmd string) error {
-	// Join the master by specifying its token
+	// Join the control plane by specifying its token
 	joinCmd = fmt.Sprintf("%s --node-name=%s", joinCmd, config.MachineName(cc, n))
 
+	if n.ControlPlane {
+		joinCmd += " --control-plane"
+		// fix kvm driver where ip address is automatically taken from the "default" network instead from the dedicated network
+		// avoid error: "error execution phase control-plane-prepare/certs: error creating PKI assets: failed to write or validate certificate "apiserver": certificate apiserver is invalid: x509: certificate is valid for 192.168.39.147, 10.96.0.1, 127.0.0.1, 10.0.0.1, 192.168.39.58, not 192.168.122.21"
+		// ref: https://kubernetes.io/docs/reference/setup-tools/kubeadm/kubeadm-join/#options
+		// "If the node should host a new control plane instance, the IP address the API Server will advertise it's listening on. If not set the default network interface will be used."
+		// "If the node should host a new control plane instance, the port for the API Server to bind to."
+		joinCmd += " --apiserver-advertise-address=" + n.IP +
+			" --apiserver-bind-port=" + strconv.Itoa(n.Port)
+	}
+
 	if _, err := k.c.RunCmd(exec.Command("/bin/bash", "-c", joinCmd)); err != nil {
 		return errors.Wrapf(err, "kubeadm join")
 	}
@@ -828,7 +785,7 @@ func (k *Bootstrapper) GenerateToken(cc config.ClusterConfig) (string, error) {
 	joinCmd = strings.Replace(joinCmd, "kubeadm", bsutil.InvokeKubeadm(cc.KubernetesConfig.KubernetesVersion), 1)
 	joinCmd = fmt.Sprintf("%s --ignore-preflight-errors=all", strings.TrimSpace(joinCmd))
 
-	// avoid "Found multiple CRI sockets, please use --cri-socket to select one: /var/run/dockershim.sock, /var/run/crio/crio.sock" error
+	// avoid "Found multiple CRI endpoints on the host. Please define which one do you wish to use by setting the 'criSocket' field in the kubeadm configuration file: unix:///var/run/containerd/containerd.sock, unix:///var/run/cri-dockerd.sock" error
 	version, err := util.ParseKubernetesVersion(cc.KubernetesConfig.KubernetesVersion)
 	if err != nil {
 		return "", errors.Wrap(err, "parsing Kubernetes version")
@@ -837,7 +794,15 @@ func (k *Bootstrapper) GenerateToken(cc config.ClusterConfig) (string, error) {
 	if err != nil {
 		klog.Errorf("cruntime: %v", err)
 	}
+
 	sp := cr.SocketPath()
+	// avoid warning/error:
+	// 'Usage of CRI endpoints without URL scheme is deprecated and can cause kubelet errors in the future.
+	//  Automatically prepending scheme "unix" to the "criSocket" with value "/var/run/cri-dockerd.sock".
+	//  Please update your configuration!'
+	if !strings.HasPrefix(sp, "unix://") {
+		sp = "unix://" + sp
+	}
 	joinCmd = fmt.Sprintf("%s --cri-socket %s", joinCmd, sp)
 
 	return joinCmd, nil
@@ -903,12 +868,14 @@ func (k *Bootstrapper) DeleteCluster(k8s config.KubernetesConfig) error {
 }
 
 // SetupCerts sets up certificates within the cluster.
-func (k *Bootstrapper) SetupCerts(k8s config.ClusterConfig, n config.Node) error {
-	return bootstrapper.SetupCerts(k.c, k8s, n)
+func (k *Bootstrapper) SetupCerts(k8s config.ClusterConfig, n config.Node, pcpCmd cruntime.CommandRunner) error {
+	return bootstrapper.SetupCerts(k8s, n, pcpCmd, k.c)
 }
 
 // UpdateCluster updates the control plane with cluster-level info.
 func (k *Bootstrapper) UpdateCluster(cfg config.ClusterConfig) error {
+	klog.Infof("updating cluster %+v ...", cfg)
+
 	images, err := images.Kubeadm(cfg.KubernetesConfig.ImageRepository, cfg.KubernetesConfig.KubernetesVersion)
 	if err != nil {
 		return errors.Wrap(err, "kubeadm images")
@@ -943,25 +910,22 @@ func (k *Bootstrapper) UpdateCluster(cfg config.ClusterConfig) error {
 		}
 	}
 
-	cp, err := config.PrimaryControlPlane(&cfg)
-	if err != nil {
-		return errors.Wrap(err, "getting control plane")
+	pcp, err := config.ControlPlane(cfg)
+	if err != nil || !config.IsPrimaryControlPlane(cfg, pcp) {
+		return errors.Wrap(err, "get primary control-plane node")
 	}
 
-	err = k.UpdateNode(cfg, cp, r)
+	err = k.UpdateNode(cfg, pcp, r)
 	if err != nil {
-		return errors.Wrap(err, "updating control plane")
+		return errors.Wrap(err, "update primary control-plane node")
 	}
 
 	return nil
 }
 
-// UpdateNode updates a node.
+// UpdateNode updates new or existing node.
 func (k *Bootstrapper) UpdateNode(cfg config.ClusterConfig, n config.Node, r cruntime.Manager) error {
-	kubeadmCfg, err := bsutil.GenerateKubeadmYAML(cfg, n, r)
-	if err != nil {
-		return errors.Wrap(err, "generating kubeadm cfg")
-	}
+	klog.Infof("updating node %v ...", n)
 
 	kubeletCfg, err := bsutil.NewKubeletConfig(cfg, n, r)
 	if err != nil {
@@ -975,19 +939,45 @@ func (k *Bootstrapper) UpdateNode(cfg config.ClusterConfig, n config.Node, r cru
 
 	klog.Infof("kubelet %s config:\n%+v", kubeletCfg, cfg.KubernetesConfig)
 
-	sm := sysinit.New(k.c)
-
-	if err := bsutil.TransferBinaries(cfg.KubernetesConfig, k.c, sm, cfg.BinaryMirror); err != nil {
-		return errors.Wrap(err, "downloading binaries")
-	}
-
 	files := []assets.CopyableFile{
 		assets.NewMemoryAssetTarget(kubeletCfg, bsutil.KubeletSystemdConfFile, "0644"),
 		assets.NewMemoryAssetTarget(kubeletService, bsutil.KubeletServiceFile, "0644"),
 	}
 
 	if n.ControlPlane {
-		files = append(files, assets.NewMemoryAssetTarget(kubeadmCfg, constants.KubeadmYamlPath+".new", "0640"))
+		// for primary control-plane node only, generate kubeadm config based on current params
+		// on node restart, it will be checked against later if anything needs changing
+		var kubeadmCfg []byte
+		if config.IsPrimaryControlPlane(cfg, n) {
+			kubeadmCfg, err = bsutil.GenerateKubeadmYAML(cfg, n, r)
+			if err != nil {
+				return errors.Wrap(err, "generating kubeadm cfg")
+			}
+			files = append(files, assets.NewMemoryAssetTarget(kubeadmCfg, constants.KubeadmYamlPath+".new", "0640"))
+		}
+		// deploy kube-vip for ha (multi-control plane) cluster
+		if config.IsHA(cfg) {
+			// workaround for kube-vip
+			// only applicable for k8s v1.29+ during primary control-plane node's kubeadm init (ie, first boot)
+			// TODO (prezha): remove when fixed upstream - ref: https://github.com/kube-vip/kube-vip/issues/684#issuecomment-1864855405
+			kv, err := semver.ParseTolerant(cfg.KubernetesConfig.KubernetesVersion)
+			if err != nil {
+				return errors.Wrapf(err, "parsing kubernetes version %q", cfg.KubernetesConfig.KubernetesVersion)
+			}
+			workaround := kv.GTE(semver.Version{Major: 1, Minor: 29}) && config.IsPrimaryControlPlane(cfg, n) && len(config.ControlPlanes(cfg)) == 1
+			kubevipCfg, err := kubevip.Configure(cfg, k.c, kubeadmCfg, workaround)
+			if err != nil {
+				klog.Errorf("couldn't generate kube-vip config, this might cause issues (will continue): %v", err)
+			} else {
+				files = append(files, assets.NewMemoryAssetTarget(kubevipCfg, path.Join(vmpath.GuestManifestsDir, kubevip.Manifest), "0600"))
+			}
+		}
+	}
+
+	sm := sysinit.New(k.c)
+
+	if err := bsutil.TransferBinaries(cfg.KubernetesConfig, k.c, sm, cfg.BinaryMirror); err != nil {
+		return errors.Wrap(err, "downloading binaries")
 	}
 
 	// Installs compatibility shims for non-systemd environments
@@ -1006,13 +996,23 @@ func (k *Bootstrapper) UpdateNode(cfg config.ClusterConfig, n config.Node, r cru
 		return errors.Wrap(err, "resolv.conf")
 	}
 
-	cp, err := config.PrimaryControlPlane(&cfg)
-	if err != nil {
-		return errors.Wrap(err, "control plane")
+	// add "control-plane.minikube.internal" dns alias
+	// note: needs to be called after APIServerHAVIP is set (in startPrimaryControlPlane()) and before kubeadm kicks off
+	cpIP := cfg.KubernetesConfig.APIServerHAVIP
+	if !config.IsHA(cfg) {
+		cp, err := config.ControlPlane(cfg)
+		if err != nil {
+			return errors.Wrap(err, "get control-plane node")
+		}
+		cpIP = cp.IP
+	}
+	if err := machine.AddHostAlias(k.c, constants.ControlPlaneAlias, net.ParseIP(cpIP)); err != nil {
+		return errors.Wrap(err, "add control-plane alias")
 	}
 
-	if err := machine.AddHostAlias(k.c, constants.ControlPlaneAlias, net.ParseIP(cp.IP)); err != nil {
-		return errors.Wrap(err, "host alias")
+	// "ensure" kubelet is started, intentionally non-fatal in case of an error
+	if err := sysinit.New(k.c).Start("kubelet"); err != nil {
+		klog.Errorf("Couldn't ensure kubelet is started this might cause issues (will continue): %v", err)
 	}
 
 	return nil
@@ -1040,37 +1040,63 @@ func kubectlPath(cfg config.ClusterConfig) string {
 	return path.Join(vmpath.GuestPersistentDir, "binaries", cfg.KubernetesConfig.KubernetesVersion, "kubectl")
 }
 
-// applyNodeLabels applies minikube labels to all the nodes
-// but it's currently called only from kubeadm.StartCluster (via kubeadm.init) where there's only one - first node
-func (k *Bootstrapper) applyNodeLabels(cfg config.ClusterConfig) error {
-	// time cluster was created. time format is based on ISO 8601 (RFC 3339)
+func (k *Bootstrapper) LabelAndUntaintNode(cfg config.ClusterConfig, n config.Node) error {
+	return k.labelAndUntaintNode(cfg, n)
+}
+
+// labelAndUntaintNode applies minikube labels to node and removes NoSchedule taints that might be set to secondary control-plane nodes by default in ha (multi-control plane) cluster.
+func (k *Bootstrapper) labelAndUntaintNode(cfg config.ClusterConfig, n config.Node) error {
+	// time node was created. time format is based on ISO 8601 (RFC 3339)
 	// converting - and : to _ because of Kubernetes label restriction
 	createdAtLbl := "minikube.k8s.io/updated_at=" + time.Now().Format("2006_01_02T15_04_05_0700")
+
 	verLbl := "minikube.k8s.io/version=" + version.GetVersion()
 	commitLbl := "minikube.k8s.io/commit=" + version.GetGitCommitID()
-	nameLbl := "minikube.k8s.io/name=" + cfg.Name
+	profileNameLbl := "minikube.k8s.io/name=" + cfg.Name
 
 	// ensure that "primary" label is applied only to the 1st node in the cluster (used eg for placing ingress there)
 	// this is used to uniquely distinguish that from other nodes in multi-master/multi-control-plane cluster config
 	primaryLbl := "minikube.k8s.io/primary=false"
-	if len(cfg.Nodes) <= 1 {
+	if config.IsPrimaryControlPlane(cfg, n) {
 		primaryLbl = "minikube.k8s.io/primary=true"
 	}
 
 	ctx, cancel := context.WithTimeout(context.Background(), applyTimeoutSeconds*time.Second)
 	defer cancel()
-	// example:
-	// sudo /var/lib/minikube/binaries/<version>/kubectl label nodes minikube.k8s.io/version=<version> minikube.k8s.io/commit=aa91f39ffbcf27dcbb93c4ff3f457c54e585cf4a-dirty minikube.k8s.io/name=p1 minikube.k8s.io/updated_at=2020_02_20T12_05_35_0700 --all --overwrite --kubeconfig=/var/lib/minikube/kubeconfig
-	cmd := exec.CommandContext(ctx, "sudo", kubectlPath(cfg),
-		"label", "nodes", verLbl, commitLbl, nameLbl, createdAtLbl, primaryLbl, "--all", "--overwrite",
-		fmt.Sprintf("--kubeconfig=%s", path.Join(vmpath.GuestPersistentDir, "kubeconfig")))
 
+	// node name is usually based on profile/cluster name, except for "none" driver where it assumes hostname
+	nodeName := config.MachineName(cfg, n)
+	if driver.IsNone(cfg.Driver) {
+		if n, err := os.Hostname(); err == nil {
+			nodeName = n
+		}
+	}
+
+	// example:
+	// sudo /var/lib/minikube/binaries/<version>/kubectl --kubeconfig=/var/lib/minikube/kubeconfig label --overwrite nodes test-357 minikube.k8s.io/version=<version> minikube.k8s.io/commit=aa91f39ffbcf27dcbb93c4ff3f457c54e585cf4a-dirty minikube.k8s.io/name=p1 minikube.k8s.io/updated_at=2020_02_20T12_05_35_0700
+	cmd := exec.CommandContext(ctx, "sudo", kubectlPath(cfg), fmt.Sprintf("--kubeconfig=%s", path.Join(vmpath.GuestPersistentDir, "kubeconfig")),
+		"label", "--overwrite", "nodes", nodeName, createdAtLbl, verLbl, commitLbl, profileNameLbl, primaryLbl)
 	if _, err := k.c.RunCmd(cmd); err != nil {
 		if ctx.Err() == context.DeadlineExceeded {
-			return errors.Wrapf(err, "timeout apply labels")
+			return errors.Wrapf(err, "timeout apply node labels")
 		}
-		return errors.Wrapf(err, "applying node labels")
+		return errors.Wrapf(err, "apply node labels")
 	}
+
+	// primary control-plane and worker nodes should be untainted by default
+	if n.ControlPlane && !config.IsPrimaryControlPlane(cfg, n) {
+		// example:
+		// sudo /var/lib/minikube/binaries/<version>/kubectl --kubeconfig=/var/lib/minikube/kubeconfig taint nodes test-357 node-role.kubernetes.io/control-plane:NoSchedule-
+		cmd := exec.CommandContext(ctx, "sudo", kubectlPath(cfg), fmt.Sprintf("--kubeconfig=%s", path.Join(vmpath.GuestPersistentDir, "kubeconfig")),
+			"taint", "nodes", config.MachineName(cfg, n), "node-role.kubernetes.io/control-plane:NoSchedule-")
+		if _, err := k.c.RunCmd(cmd); err != nil {
+			if ctx.Err() == context.DeadlineExceeded {
+				return errors.Wrapf(err, "timeout remove node taints")
+			}
+			return errors.Wrapf(err, "remove node taints")
+		}
+	}
+
 	return nil
 }
 
@@ -1078,7 +1104,7 @@ func (k *Bootstrapper) applyNodeLabels(cfg config.ClusterConfig) error {
 func (k *Bootstrapper) elevateKubeSystemPrivileges(cfg config.ClusterConfig) error {
 	start := time.Now()
 	defer func() {
-		klog.Infof("duration metric: took %s to wait for elevateKubeSystemPrivileges.", time.Since(start))
+		klog.Infof("duration metric: took %s to wait for elevateKubeSystemPrivileges", time.Since(start))
 	}()
 
 	// Allow no more than 5 seconds for creating cluster role bindings
