@@ -23,6 +23,7 @@ import (
 	"sort"
 	"strconv"
 	"strings"
+	"time"
 
 	"golang.org/x/text/cases"
 	"golang.org/x/text/language"
@@ -354,12 +355,25 @@ func Suggest(options []registry.DriverState) (registry.DriverState, []registry.D
 // Status returns the status of a driver
 func Status(name string) registry.DriverState {
 	d := registry.Driver(name)
-	return registry.DriverState{
-		Name:     d.Name,
-		Default:  d.Default,
-		Priority: d.Priority,
-		State:    registry.Status(name),
+	stateChannel := make(chan registry.State)
+	timeoutChannel := time.After(20 * time.Second)
+	go func() {
+		stateChannel <- registry.Status(name)
+	}()
+	select {
+	case s := <-stateChannel:
+		return registry.DriverState{
+			Name:     d.Name,
+			Default:  d.Default,
+			Priority: d.Priority,
+			State:    s,
+		}
+	case <-timeoutChannel:
+		klog.Infof("time out when checking for status of %s driver", name)
+		return registry.DriverState{}
+
 	}
+
 }
 
 // IsAlias checks if an alias belongs to provided driver by name.
