@@ -27,6 +27,7 @@ var supportedDrivers = func() []string {
 	if runtime.GOARCH == "arm64" {
 		// on darwin/arm64 only docker and ssh are supported yet
 		return []string{
+			QEMU2,
 			Docker,
 			Podman,
 			SSH,
@@ -37,7 +38,6 @@ var supportedDrivers = func() []string {
 		return []string{
 			VirtualBox,
 			Parallels,
-			VMwareFusion,
 			HyperKit,
 			VMware,
 			Docker,
@@ -47,9 +47,9 @@ var supportedDrivers = func() []string {
 	return []string{
 		VirtualBox,
 		Parallels,
-		VMwareFusion,
 		HyperKit,
 		VMware,
+		QEMU2,
 		Docker,
 		Podman,
 		SSH,
