@@ -217,6 +217,12 @@ var (
 		Style: style.UnmetRequirement,
 		URL:   "https://docs.docker.com/docker-for-mac/#resources",
 	}
+	// invalid memory value for Hyper-V
+	RsrcInvalidHyperVMemory = Kind{
+		ID:       "RSRC_INVALID_HYPERV_MEMORY",
+		ExitCode: ExResourceError,
+		Style:    style.UnmetRequirement,
+	}
 
 	// insufficient disk storage available to the docker driver
 	RsrcInsufficientDockerStorage = Kind{
@@ -475,6 +481,8 @@ var (
 	KubernetesTooOld = Kind{ID: "K8S_OLD_UNSUPPORTED", ExitCode: ExControlPlaneUnsupported}
 	// a too new Kubernetes version was specified for minikube to use
 	KubernetesTooNew = Kind{ID: "K8S_NEW_UNSUPPORTED", ExitCode: ExControlPlaneUnsupported}
+	// error fetching GitHub Kubernetes version list
+	KubernetesNotConnect = Kind{ID: "K8S_FAIL_CONNECT", ExitCode: ExInternetError}
 	// minikube was unable to safely downgrade installed Kubernetes version
 	KubernetesDowngrade = Kind{
 		ID:       "K8S_DOWNGRADE_UNSUPPORTED",
