@@ -41,6 +41,16 @@ func TestMultiNode(t *testing.T) {
 		t.Skip("none driver does not support multinode")
 	}
 
+	if DockerDriver() {
+		rr, err := Run(t, exec.Command("docker", "version", "-f", "{{.Server.Version}}"))
+		if err != nil {
+			t.Fatalf("docker is broken: %v", err)
+		}
+		if strings.Contains(rr.Stdout.String(), "azure") {
+			t.Skip("kic containers are not supported on docker's azure")
+		}
+	}
+
 	type validatorFunc func(context.Context, *testing.T, string)
 	profile := UniqueProfileName("multinode")
 	ctx, cancel := context.WithTimeout(context.Background(), Minutes(30))
@@ -55,6 +65,7 @@ func TestMultiNode(t *testing.T) {
 			{"DeployApp2Nodes", validateDeployAppToMultiNode},
 			{"PingHostFrom2Pods", validatePodsPingHost},
 			{"AddNode", validateAddNodeToMultiNode},
+			{"MultiNodeLabels", validateMultiNodeLabels},
 			{"ProfileList", validateProfileListWithMultiNode},
 			{"CopyFile", validateCopyFileWithMultiNode},
 			{"StopNode", validateStopRunningNode},
@@ -164,7 +175,7 @@ func validateProfileListWithMultiNode(ctx context.Context, t *testing.T, profile
 	}
 }
 
-// validateProfileListWithMultiNode make sure minikube profile list outputs correct with multinode clusters
+// validateCopyFileWithMultiNode make sure minikube cp works with multinode clusters.
 func validateCopyFileWithMultiNode(ctx context.Context, t *testing.T, profile string) {
 	if NoneDriver() {
 		t.Skipf("skipping: cp is unsupported by none driver")
@@ -204,6 +215,33 @@ func validateCopyFileWithMultiNode(ctx context.Context, t *testing.T, profile st
 	}
 }
 
+// validateMultiNodeLabels check if all node labels were configured correctly
+func validateMultiNodeLabels(ctx context.Context, t *testing.T, profile string) {
+	// docs: Get the node labels from the cluster with `kubectl get nodes`
+	rr, err := Run(t, exec.CommandContext(ctx, "kubectl", "--context", profile, "get", "nodes", "-o", "jsonpath=[{range .items[*]}{.metadata.labels},{end}]"))
+	if err != nil {
+		t.Errorf("failed to 'kubectl get nodes' with args %q: %v", rr.Command(), err)
+	}
+
+	nodeLabelsList := []map[string]string{}
+	fixedString := strings.Replace(rr.Stdout.String(), ",]", "]", 1)
+	err = json.Unmarshal([]byte(fixedString), &nodeLabelsList)
+	if err != nil {
+		t.Errorf("failed to decode json from label list: args %q: %v", rr.Command(), err)
+	}
+
+	// docs: check if all node labels matches with the expected Minikube labels: `minikube.k8s.io/*`
+	expectedLabels := []string{"minikube.k8s.io/commit", "minikube.k8s.io/version", "minikube.k8s.io/updated_at", "minikube.k8s.io/name", "minikube.k8s.io/primary"}
+
+	for _, nodeLabels := range nodeLabelsList {
+		for _, el := range expectedLabels {
+			if _, ok := nodeLabels[el]; !ok {
+				t.Errorf("expected to have label %q in node labels but got : %s", el, rr.Output())
+			}
+		}
+	}
+}
+
 // validateStopRunningNode tests the minikube node stop command
 func validateStopRunningNode(ctx context.Context, t *testing.T, profile string) {
 	// Run minikube node stop on that node
@@ -240,26 +278,19 @@ func validateStopRunningNode(ctx context.Context, t *testing.T, profile string)
 
 // validateStartNodeAfterStop tests the minikube node start command on an existing stopped node
 func validateStartNodeAfterStop(ctx context.Context, t *testing.T, profile string) {
-	if DockerDriver() {
-		rr, err := Run(t, exec.Command("docker", "version", "-f", "{{.Server.Version}}"))
-		if err != nil {
-			t.Fatalf("docker is broken: %v", err)
-		}
-		if strings.Contains(rr.Stdout.String(), "azure") {
-			t.Skip("kic containers are not supported on docker's azure")
-		}
-	}
-
 	// Start the node back up
-	rr, err := Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "node", "start", ThirdNodeName, "--alsologtostderr"))
+	rr, err := Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "node", "start", ThirdNodeName, "-v=7", "--alsologtostderr"))
 	if err != nil {
 		t.Logf(rr.Stderr.String())
 		t.Errorf("node start returned an error. args %q: %v", rr.Command(), err)
 	}
 
 	// Make sure minikube status shows 3 running hosts
-	rr, err = Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "status"))
-	if err != nil {
+	minikubeStatus := func() error {
+		rr, err = Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "status", "-v=7", "--alsologtostderr"))
+		return err
+	}
+	if err := retry.Expo(minikubeStatus, 1*time.Second, 60*time.Second); err != nil {
 		t.Fatalf("failed to run minikube status. args %q : %v", rr.Command(), err)
 	}
 
@@ -313,7 +344,7 @@ func validateStopMultiNodeCluster(ctx context.Context, t *testing.T, profile str
 	// Run minikube stop on the cluster
 	rr, err := Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "stop"))
 	if err != nil {
-		t.Errorf("node stop returned an error. args %q: %v", rr.Command(), err)
+		t.Errorf("failed to stop cluster. args %q: %v", rr.Command(), err)
 	}
 
 	// Run status to see the stopped hosts
@@ -340,15 +371,6 @@ func validateStopMultiNodeCluster(ctx context.Context, t *testing.T, profile str
 
 // validateRestartMultiNodeCluster verifies a soft restart on a multinode cluster works
 func validateRestartMultiNodeCluster(ctx context.Context, t *testing.T, profile string) {
-	if DockerDriver() {
-		rr, err := Run(t, exec.Command("docker", "version", "-f", "{{.Server.Version}}"))
-		if err != nil {
-			t.Fatalf("docker is broken: %v", err)
-		}
-		if strings.Contains(rr.Stdout.String(), "azure") {
-			t.Skip("kic containers are not supported on docker's azure")
-		}
-	}
 	// Restart a full cluster with minikube start
 	startArgs := append([]string{"start", "-p", profile, "--wait=true", "-v=8", "--alsologtostderr"}, StartArgs()...)
 	rr, err := Run(t, exec.CommandContext(ctx, Target(), startArgs...))
@@ -390,10 +412,10 @@ func validateRestartMultiNodeCluster(ctx context.Context, t *testing.T, profile
 
 // validateDeleteNodeFromMultiNode tests the minikube node delete command
 func validateDeleteNodeFromMultiNode(ctx context.Context, t *testing.T, profile string) {
-	// Start the node back up
+	// Delete a node from the current cluster
 	rr, err := Run(t, exec.CommandContext(ctx, Target(), "-p", profile, "node", "delete", ThirdNodeName))
 	if err != nil {
-		t.Errorf("node stop returned an error. args %q: %v", rr.Command(), err)
+		t.Errorf("node delete returned an error. args %q: %v", rr.Command(), err)
 	}
 
 	// Make sure status is back down to 2 hosts
@@ -410,16 +432,6 @@ func validateDeleteNodeFromMultiNode(ctx context.Context, t *testing.T, profile
 		t.Errorf("status says both kubelets are not running: args %q: %v", rr.Command(), rr.Stdout.String())
 	}
 
-	if DockerDriver() {
-		rr, err := Run(t, exec.Command("docker", "volume", "ls"))
-		if err != nil {
-			t.Errorf("failed to run %q : %v", rr.Command(), err)
-		}
-		if strings.Contains(rr.Stdout.String(), fmt.Sprintf("%s-%s", profile, ThirdNodeName)) {
-			t.Errorf("docker volume was not properly deleted: %s", rr.Stdout.String())
-		}
-	}
-
 	// Make sure kubectl knows the node is gone
 	rr, err = Run(t, exec.CommandContext(ctx, "kubectl", "get", "nodes"))
 	if err != nil {
